﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IGCheckboxTree
{
    /// <summary>
    /// Class defines logic for a Differences tree item (actual difference will have zero children)
    /// </summary>
    public class Item : NotificationObject
    {
        /// <summary>
        /// Ctor
        /// </summary>
        /// <param name="name">Name of the node/item</param>
        public Item(string name, bool isEnabled, bool initialState)
        {
            Name = name;
            this.Children = new ObservableCollection<Item>();
            Children.CollectionChanged += Children_CollectionChanged;
            this.IsChecked = initialState;
            this.IsEnabled = isEnabled;
        }

        /// <summary>
        /// Initialize object (the top most node)
        /// </summary>
        public void Initialize()
        {
            foreach (Item child in this.Children)
            {
                child.Parent = this;
                child.Initialize();
            }

            if (this.Children.Count > 0)
            {
                VerifyCheckState();
            }
        }

        /// <summary>
        /// The item's name
        /// </summary>
        public string Name { get; private set; }

        public bool IsEnabled { get; set; }

        /// <summary>
        /// The parent item of the item (gets set when Initialized)
        /// </summary>
        Item Parent;

        /// <summary>
        /// Items collection of children items
        /// </summary>
        public ObservableCollection<Item> Children { get; private set; }

        ///// <summary>
        ///// Bool for if item is Initially Selected
        ///// </summary>
        //public bool IsInitiallySelected { get; set; }

        private bool? _isChecked;
        /// <summary>
        /// Bool for whether the Item is checked (true) or not (false) 
        /// </summary>
        public bool? IsChecked
        {
            get { return _isChecked; }
            //set
            //{
            //    if (_isChecked != value)
            //    {
            //        _isChecked = value;
            //        FirePropertyChanged(() => IsChecked);
            //        FirePropertyChanged(() => Accepted);
            //    }
            //}
            set { this.SetIsChecked(value, true, true); }
        }

        void SetIsChecked(bool? value, bool updateChildren, bool updateParent)
        {
            if (_isChecked != value)
            {
                _isChecked = value;

                //if (updateChildren && _isChecked.HasValue)
                //{
                //    foreach (Item c in this.Children)
                //    {
                //        c.SetIsChecked(_isChecked, true, false);
                //    }
                //}

                //if (updateParent && Parent != null)
                //{
                //    Parent.VerifyCheckState();
                //}
                
                FirePropertyChanged(() => IsChecked);
                FirePropertyChanged(() => Accepted);
            }
        }

        /// <summary>
        /// Verifies the check state
        /// </summary>
        private void VerifyCheckState()
        {
            bool? state = null;
            for (int i = 0; i < this.Children.Count; ++i)
            {
                bool? current = this.Children[i].IsChecked;
                if (i == 0)
                {
                    state = current;
                }
                else if (state != current)
                {
                    state = null;
                    break;
                }
            }
            this.SetIsChecked(state, false, true);
        }

        /// <summary>
        /// Number of accepted differences in the items collection (or itself if no children)
        /// </summary>
        public int Accepted
        {
            get { return CountTotalAccepted(this); }
        }

        /// <summary>
        /// Total number of differences in the items collection (or itself if no children)
        /// </summary>
        public int Differences
        {
            get { return CountTotalDifferences(this); }
        }

       

        /// <summary>
        /// Count the total number of difference items within
        /// </summary>
        /// <param name="collection"></param>
        /// <returns></returns>
        public int CountTotalDifferences(Item node)
        {
            int count = 0;

            if (node.Children.Count != 0)
            {
                for (int i = 0; i < node.Children.Count; i++)
                {
                    if (node.Children[i].Children.Count == 0)
                    {
                        count++;
                    }
                    else
                    {
                        count += CountTotalDifferences(node.Children[i]);
                    }
                }
                //foreach (Item item in node.Children)
                //{
                //    if (item.Children.Count == 0)
                //    {
                //        if (item._isChecked == true)
                //        {
                //            count++;
                //        }
                //    }
                //    else
                //    {
                //        count += CountTotalDifferences(item);
                //    }
                //}
            }
            else
            {
                count = 1;
            }

            return count;
        }

        /// <summary>
        /// Count the number of differences that are set to IsChecked within
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public int CountTotalAccepted(Item node)
        {
            int count = 0;

            if (node.Children.Count != 0)
            {
                for (int i = 0; i < node.Children.Count; i++)
                {
                    if (node.Children[i].Children.Count == 0)
                    {
                        if (node.Children[i].IsChecked == true)
                        {
                            count++;
                        }
                    }
                    else
                    {
                        count += CountTotalAccepted(node.Children[i]);
                    }
                }
            }

            //if (node.Children.Count != 0)
            //{
            //    foreach (Item item in node.Children)
            //    {
            //        if (item.Children.Count == 0)
            //        {
            //            if (item.IsChecked == true)
            //            {
            //                count++;
            //            }
            //            //else
            //            //{
            //            //    //count--;
            //            //}
            //        }
            //        else
            //        {
            //            count += CountTotalAccepted(item);
            //        }
            //    }
            //}
            else
            {
                if (IsChecked == true)
                {
                    count = 1;
                }
                else
                {
                    count = 0;
                }
            }

            return count;
        }

        /// <summary>
        /// Collection changed notification
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Children_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null)
            {
                foreach (Item item in e.NewItems)
                {
                    item.PropertyChanged += Item_PropertyChanged;
                }
            }
            if (e.OldItems != null)
            {
                foreach (Item item in e.OldItems)
                {
                    item.PropertyChanged -= Item_PropertyChanged;
                }
            }
        }

        /// <summary>
        /// Updates the Accepted property anytime collection has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            FirePropertyChanged(() => Accepted);
        }
    }
}
